@@ -34,6 +34,11 @@ import ldap
 import logging
 import pprint
 
+try:
+    from django.utils.encoding import force_str
+except ImportError:  # Django < 1.5
+    from django.utils.encoding import smart_str as force_str
+
 
 class _LDAPConfig(object):
     """
@@ -65,7 +70,7 @@ class _LDAPConfig(object):
 
         # Apply global LDAP options once
         if (not cls._ldap_configured) and (global_options is not None):
-            for opt, value in global_options.iteritems():
+            for opt, value in global_options.items():
                 cls.ldap.set_option(opt, value)
 
             cls._ldap_configured = True
@@ -118,7 +123,7 @@ class LDAPSearch(object):
         """
         term_strings = [self.filterstr]
 
-        for name, value in term_dict.iteritems():
+        for name, value in term_dict.items():
             if escape:
                 value = self.ldap.filter.escape_filter_chars(value)
             term_strings.append(u'(%s=%s)' % (name, value))
@@ -137,37 +142,48 @@ class LDAPSearch(object):
 
         return self.__class__(self.base_dn, self.scope, filterstr)
 
-    def execute(self, connection, filterargs=()):
+    def execute(self, connection, filterargs=(), escape=True):
         """
         Executes the search on the given connection (an LDAPObject). filterargs
         is an object that will be used for expansion of the filter string.
+        If escape is True, values in filterargs will be escaped.
 
         The python-ldap library returns utf8-encoded strings. For the sake of
         sanity, this method will decode all result strings and return them as
         Unicode.
         """
+        if escape:
+            filterargs = self._escape_filterargs(filterargs)
+
         try:
             filterstr = self.filterstr % filterargs
-            results = connection.search_s(self.base_dn.encode('utf-8'),
+            results = connection.search_s(force_str(self.base_dn),
                                           self.scope,
-                                          filterstr.encode('utf-8'))
-        except ldap.LDAPError, e:
+                                          force_str(filterstr))
+        except ldap.LDAPError as e:
             results = []
             logger.error(u"search_s('%s', %d, '%s') raised %s" %
                          (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
 
         return self._process_results(results)
 
-    def _begin(self, connection, filterargs=()):
+    def _begin(self, connection, filterargs=(), escape=True):
         """
         Begins an asynchronous search and returns the message id to retrieve
         the results.
+
+        filterargs is an object that will be used for expansion of the filter
+        string. If escape is True, values in filterargs will be escaped.
+
         """
+        if escape:
+            filterargs = self._escape_filterargs(filterargs)
+
         try:
             filterstr = self.filterstr % filterargs
-            msgid = connection.search(self.base_dn.encode('utf-8'),
-                                      self.scope, filterstr.encode('utf-8'))
-        except ldap.LDAPError, e:
+            msgid = connection.search(force_str(self.base_dn),
+                                      self.scope, force_str(filterstr))
+        except ldap.LDAPError as e:
             msgid = None
             logger.error(u"search('%s', %d, '%s') raised %s" %
                          (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
@@ -182,22 +198,42 @@ class LDAPSearch(object):
             kind, results = connection.result(msgid)
             if kind != ldap.RES_SEARCH_RESULT:
                 results = []
-        except ldap.LDAPError, e:
+        except ldap.LDAPError as e:
             results = []
             logger.error(u"result(%d) raised %s" % (msgid, pprint.pformat(e)))
 
         return self._process_results(results)
 
+    def _escape_filterargs(self, filterargs):
+        """
+        Escapes values in filterargs.
+
+        filterargs is a value suitable for Django's string formatting operator
+        (%), which means it's either a tuple or a dict. This return a new tuple
+        or dict with all values escaped for use in filter strings.
+
+        """
+        if isinstance(filterargs, tuple):
+            filterargs = tuple(self.ldap.filter.escape_filter_chars(value)
+                               for value in filterargs)
+        elif isinstance(filterargs, dict):
+            filterargs = dict((key, self.ldap.filter.escape_filter_chars(value))
+                              for key, value in filterargs.items())
+        else:
+            raise TypeError("filterargs must be a tuple or dict.")
+
+        return filterargs
+
     def _process_results(self, results):
         """
         Returns a sanitized copy of raw LDAP results. This scrubs out
         references, decodes utf8, normalizes DNs, etc.
         """
-        results = filter(lambda r: r[0] is not None, results)
+        results = [r for r in results if r[0] is not None]
         results = _DeepStringCoder('utf-8').decode(results)
 
         # The normal form of a DN is lower case.
-        results = map(lambda r: (r[0].lower(), r[1]), results)
+        results = [(r[0].lower(), r[1]) for r in results]
 
         result_dns = [result[0] for result in results]
         logger.debug(u"search_s('%s', %d, '%s') returned %d objects: %s" %
@@ -233,8 +269,9 @@ class LDAPSearchUnion(object):
         results = {}
 
         for search, msgid in zip(self.searches, msgids):
-            result = search._results(connection, msgid)
-            results.update(dict(result))
+            if msgid is not None:
+                result = search._results(connection, msgid)
+                results.update(dict(result))
 
         return results.items()
 
@@ -251,7 +288,7 @@ class _DeepStringCoder(object):
 
     def decode(self, value):
         try:
-            if isinstance(value, str):
+            if isinstance(value, bytes):
                 value = value.decode(self.encoding)
             elif isinstance(value, list):
                 value = self._decode_list(value)
@@ -273,7 +310,7 @@ class _DeepStringCoder(object):
         # for search results.
         decoded = self.ldap.cidict.cidict()
 
-        for k, v in value.iteritems():
+        for k, v in value.items():
             decoded[self.decode(k)] = self.decode(v)
 
         return decoded
@@ -386,14 +423,14 @@ class PosixGroupType(LDAPGroupType):
             user_uid = ldap_user.attrs['uid'][0]
 
             try:
-                is_member = ldap_user.connection.compare_s(group_dn.encode('utf-8'), 'memberUid', user_uid.encode('utf-8'))
+                is_member = ldap_user.connection.compare_s(force_str(group_dn), 'memberUid', force_str(user_uid))
             except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                 is_member = False
 
             if not is_member:
                 try:
                     user_gid = ldap_user.attrs['gidNumber'][0]
-                    is_member = ldap_user.connection.compare_s(group_dn.encode('utf-8'), 'gidNumber', user_gid.encode('utf-8'))
+                    is_member = ldap_user.connection.compare_s(force_str(group_dn), 'gidNumber', force_str(user_gid))
                 except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                     is_member = False
         except (KeyError, IndexError):
@@ -424,9 +461,9 @@ class MemberDNGroupType(LDAPGroupType):
     def is_member(self, ldap_user, group_dn):
         try:
             result = ldap_user.connection.compare_s(
-                group_dn.encode('utf-8'),
-                self.member_attr.encode('utf-8'),
-                ldap_user.dn.encode('utf-8')
+                force_str(group_dn),
+                force_str(self.member_attr),
+                force_str(ldap_user.dn)
             )
         except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
             result = 0
